// src/index.ts
function generateTheme(selectorScope, theme) {
  const styles = Object.entries(selectorScope).map(([selector, VSCodeScope]) => {
    const style = parseVSCodeScopeStyle(VSCodeScope, theme);
    return { selector, style };
  }).filter((css) => css.style);
  const combinedStyles = styles.reduce((all, style) => {
    if (all[style.style]) {
      all[style.style] += `,
${style.selector}`;
    } else {
      all[style.style] = style.selector;
    }
    return all;
  }, {});
  const tokenCSS = Object.entries(combinedStyles).map(([style, selector]) => `${selector} ${style}`).join("\n");
  const global = parseDefaultThemeSetting(theme);
  const wrapperCSS = `pre code {
  display: block;
  color: ${global.foreground};
  background: ${global.background};
}`;
  return wrapperCSS + "\n" + tokenCSS;
}
function parseVSCodeScopeStyle(scope, theme) {
  const tokenMatchScores = theme.tokenColors.filter((token) => token.scope).map((token) => ({
    token,
    score: scoreScopeMatch(scope, token.scope)
  }));
  const maxMatchedToken = tokenMatchScores.reduce((all, cur) => cur.score > all.score ? cur : all, tokenMatchScores[0]);
  return maxMatchedToken.score ? formatVSCodeTokenStyle(maxMatchedToken.token.settings) : null;
}
function scoreScopeMatch(sourceScope, targetScope) {
  if (Array.isArray(targetScope)) {
    const isExactlyMathched = targetScope.some((s) => s === sourceScope);
    if (isExactlyMathched) {
      return Infinity;
    } else {
      const scores = targetScope.map((s) => scoreScopeMatch(sourceScope, s));
      const maxScore = scores.reduce((all, cur) => Math.max(all, cur), 0);
      return maxScore;
    }
  } else if (!targetScope) {
    return 0;
  } else {
    if (sourceScope === targetScope) {
      return Infinity;
    } else {
      const sourceTokens = sourceScope.split(".");
      const targetTokens = targetScope.split(".");
      let score = 0;
      while (sourceTokens.length && targetTokens.length) {
        const sourceToken = sourceTokens.shift();
        const targetToken = targetTokens.shift();
        if (sourceToken === targetToken) {
          score += 1;
        } else {
          break;
        }
      }
      return score;
    }
  }
}
function formatVSCodeTokenStyle(settings) {
  const mappings = [
    ["color", settings.foreground],
    ["background", settings.background],
    ["font-style", settings.fontStyle]
  ];
  const styles = mappings.filter((mapping) => mapping[1]).map((mapping) => `  ${mapping[0]}: ${mapping[1]};`).join("\n");
  return styles ? `{
${styles}
}` : null;
}
var VSCODE_FALLBACK_EDITOR_FG = { light: "#333333", dark: "#bbbbbb" };
var VSCODE_FALLBACK_EDITOR_BG = { light: "#fffffe", dark: "#1e1e1e" };
function parseDefaultThemeSetting(theme) {
  const settings = theme.settings || theme.tokenColors;
  const globalSetting = settings == null ? void 0 : settings.find((s) => !s.scope);
  const foreground = parseDefaultColor("foreground");
  const background = parseDefaultColor("background");
  return { foreground, background };
  function parseDefaultColor(key) {
    var _a, _b;
    const VSCODE_FALLBACK_SETTING = key === "foreground" ? VSCODE_FALLBACK_EDITOR_FG : VSCODE_FALLBACK_EDITOR_BG;
    const VSCODE_FALLBACK_COLOR = theme.type === "light" ? VSCODE_FALLBACK_SETTING.light : VSCODE_FALLBACK_SETTING.dark;
    return ((_a = globalSetting == null ? void 0 : globalSetting.settings) == null ? void 0 : _a[key]) || ((_b = theme.colors) == null ? void 0 : _b[`editor.${key}`]) || VSCODE_FALLBACK_COLOR;
  }
}

export {
  generateTheme
};
