"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/prism-js.ts
var prism_js_exports = {};
__export(prism_js_exports, {
  generatePrismjsTheme: () => generatePrismjsTheme
});
module.exports = __toCommonJS(prism_js_exports);

// src/index.ts
function generateTheme(selectorScope, theme) {
  const styles = Object.entries(selectorScope).map(([selector, VSCodeScope]) => {
    const style = parseVSCodeScopeStyle(VSCodeScope, theme);
    return { selector, style };
  }).filter((css) => css.style);
  const combinedStyles = styles.reduce((all, style) => {
    if (all[style.style]) {
      all[style.style] += `,
${style.selector}`;
    } else {
      all[style.style] = style.selector;
    }
    return all;
  }, {});
  const tokenCSS = Object.entries(combinedStyles).map(([style, selector]) => `${selector} ${style}`).join("\n");
  const global = parseDefaultThemeSetting(theme);
  const wrapperCSS = `pre code {
  display: block;
  color: ${global.foreground};
  background: ${global.background};
}`;
  return wrapperCSS + "\n" + tokenCSS;
}
function parseVSCodeScopeStyle(scope, theme) {
  const tokenMatchScores = theme.tokenColors.filter((token) => token.scope).map((token) => ({
    token,
    score: scoreScopeMatch(scope, token.scope)
  }));
  const maxMatchedToken = tokenMatchScores.reduce((all, cur) => cur.score > all.score ? cur : all, tokenMatchScores[0]);
  return maxMatchedToken.score ? formatVSCodeTokenStyle(maxMatchedToken.token.settings) : null;
}
function scoreScopeMatch(sourceScope, targetScope) {
  if (Array.isArray(targetScope)) {
    const isExactlyMathched = targetScope.some((s) => s === sourceScope);
    if (isExactlyMathched) {
      return Infinity;
    } else {
      const scores = targetScope.map((s) => scoreScopeMatch(sourceScope, s));
      const maxScore = scores.reduce((all, cur) => Math.max(all, cur), 0);
      return maxScore;
    }
  } else if (!targetScope) {
    return 0;
  } else {
    if (sourceScope === targetScope) {
      return Infinity;
    } else {
      const sourceTokens = sourceScope.split(".");
      const targetTokens = targetScope.split(".");
      let score = 0;
      while (sourceTokens.length && targetTokens.length) {
        const sourceToken = sourceTokens.shift();
        const targetToken = targetTokens.shift();
        if (sourceToken === targetToken) {
          score += 1;
        } else {
          break;
        }
      }
      return score;
    }
  }
}
function formatVSCodeTokenStyle(settings) {
  const mappings = [
    ["color", settings.foreground],
    ["background", settings.background],
    ["font-style", settings.fontStyle]
  ];
  const styles = mappings.filter((mapping) => mapping[1]).map((mapping) => `  ${mapping[0]}: ${mapping[1]};`).join("\n");
  return styles ? `{
${styles}
}` : null;
}
var VSCODE_FALLBACK_EDITOR_FG = { light: "#333333", dark: "#bbbbbb" };
var VSCODE_FALLBACK_EDITOR_BG = { light: "#fffffe", dark: "#1e1e1e" };
function parseDefaultThemeSetting(theme) {
  const settings = theme.settings || theme.tokenColors;
  const globalSetting = settings == null ? void 0 : settings.find((s) => !s.scope);
  const foreground = parseDefaultColor("foreground");
  const background = parseDefaultColor("background");
  return { foreground, background };
  function parseDefaultColor(key) {
    var _a, _b;
    const VSCODE_FALLBACK_SETTING = key === "foreground" ? VSCODE_FALLBACK_EDITOR_FG : VSCODE_FALLBACK_EDITOR_BG;
    const VSCODE_FALLBACK_COLOR = theme.type === "light" ? VSCODE_FALLBACK_SETTING.light : VSCODE_FALLBACK_SETTING.dark;
    return ((_a = globalSetting == null ? void 0 : globalSetting.settings) == null ? void 0 : _a[key]) || ((_b = theme.colors) == null ? void 0 : _b[`editor.${key}`]) || VSCODE_FALLBACK_COLOR;
  }
}

// src/prism-js.ts
var PRISMJS_SCOPE_MAPPINGS = {
  // Prism.js 中的关键字对应于 TextMate 中的关键字
  ".token.keyword": "keyword",
  // Prism.js 中的运算符对应于 TextMate 中的运算符
  ".token.operator": "keyword.operator",
  // Prism.js 中的标点符号对应于 TextMate 中的标点符号
  ".token.punctuation": "punctuation",
  // Prism.js 中的注释对应于 TextMate 中的注释
  ".token.comment": "comment",
  // Prism.js 中的字符串对应于 TextMate 中的字符串
  ".token.string": "string",
  // Prism.js 中的数字对应于 TextMate 中的数字
  ".token.number": "constant.numeric",
  // Prism.js 中的布尔值对应于 TextMate 中的常量
  ".token.boolean": "constant.language",
  // Prism.js 中的函数名对应于 TextMate 中的函数名
  ".token.function": "entity.name.function",
  // Prism.js 中的变量名对应于 TextMate 中的变量名
  ".token.variable": "variable",
  // Prism.js 中的属性名对应于 TextMate 中的属性名
  ".token.property": "support.type.property-name",
  // Prism.js 中的常量名对应于 TextMate 中的常量名
  ".token.constant": "constant",
  // Prism.js 中的类名对应于 TextMate 中的类名
  ".token.class-name": "entity.name.class",
  // Prism.js 中的 CSS 选择器对应于 TextMate 中的支持类型
  ".token.selector": "support.type",
  // Prism.js 中的 at-rule 对应于 TextMate 中的关键字
  ".token.atrule": "keyword.control.at-rule",
  // Prism.js 中的 HTML 属性名对应于 TextMate 中的属性名
  ".token.attr-name": "entity.other.attribute-name",
  // Prism.js 中的 HTML 属性值对应于 TextMate 中的字符串
  ".token.attr-value": "string",
  // Prism.js 中的 HTML 标签名对应于 TextMate 中的实体名
  ".token.tag": "entity.name.tag",
  // 其他可能的映射
  ".token.namespace": "entity.name.namespace",
  ".token.regex": "string.regexp",
  ".token.important": "keyword.other.important",
  ".token.builtin": "support",
  ".token.url": "markup.underline.link",
  ".token.symbol": "constant.other.symbol",
  ".token.module": "support.other.module",
  ".token.type": "entity.name.type",
  ".token.parameter": "variable.parameter",
  ".token.label": "entity.name.label",
  ".token.directive": "keyword.control.directive",
  ".token.inline": "markup.inline"
};
function generatePrismjsTheme(VSCodeTheme) {
  const prismjsSelectorScope = generatePrismjsSelectorScope();
  const theme = generateTheme(prismjsSelectorScope, VSCodeTheme);
  return theme;
}
function generatePrismjsSelectorScope() {
  const selectorScope = {};
  Object.entries(PRISMJS_SCOPE_MAPPINGS).forEach(
    ([prismjsScope, VSCodeScope]) => selectorScope[prismjsScope] = VSCodeScope
  );
  return selectorScope;
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  generatePrismjsTheme
});
